#include <xc.h>
#include "main.h"
#include "sound.h"
#include "frq.h"

unsigned long frq;
unsigned char frq_fine=0;
static unsigned long frq_old;

void FRQ_Init(void) {

    // Enable the NCO
    NCO1CON = 0b10000000;
    NCO1CLK = 0b00000001;
    RC3PPS = 0x03;
    TRISC = 0b11110111;

    frq = 3500000;//6200; //49m band
    frq_old = frq;
    FRQ_Update();
}

void FRQ_Update(void){
    unsigned long inc;

    //    inc = 2 * frq * 1kHz / 32MHz * 2E20 / 10;
//    inc = ((unsigned long)frq * (unsigned long)8192) / 125; // 1kHz steps
//    inc = ((unsigned long)frq * (unsigned long)4096) / 625 ; // 100Hz steps
    inc = ((unsigned long)frq * (unsigned long)1024) / 15625 ; // 100Hz steps

    // Update the NCO
    NCO1INCU = (inc >> 16) & 0x0000FF;
    NCO1INCH = (inc >> 8 ) & 0x0000FF;
    NCO1INCL = (inc >> 0 ) & 0x0000FF;
}


void FRQ_IncDec(T_INC_DEC incdec) {
    if (incdec == INC) {
        if (frq < 3800000) {
            if (frq_fine)
                frq += 16; // 10Hz steps
            else
                frq += 1000; // 1kHz steps
        }
    } else {
        if (frq > 3500000) {
            if (frq_fine)
                frq -= 16; // 10Hz steps
            else
                frq -= 1000; // 1kHz steps
        }
    }

    // Beep, if frequency has changed
    if (frq != frq_old) {
        FRQ_Update();
        if (frq_fine)
            SND_Beep(BEEP_TIC_FINE);
        else
            SND_Beep(BEEP_TIC);
    }

    frq_old = frq;
}

void FRQ_Output (void) {
    SND_Morse(frq);
}

void FRQ_ToggleFine(void) {
    frq_fine = !frq_fine;
    if (frq_fine)
        SND_Beep(BEEP_UP);
    else
        SND_Beep(BEEP_DOWN);
}