#include <xc.h>
#include "main.h"
#include "sound.h"

// local function
static void SND_MorseCode(unsigned char number);
const unsigned char SINTAB[32] = {0, 2, 10, 21, 37, 56, 78, 102, 127, 152, 176, 198, 217,
    233, 244, 252, 254, 252, 244, 233, 217, 198, 176, 152, 127, 102, 78, 56, 37, 21, 10, 2};

void SND_Init(void) {
    DAC1CON0 = 0b10100000;
    __delay_ms(1000);
    SND_Beep(BEEP_WELCOME);
}

//short beep

void SND_Beep(beepType beeptype) {
    unsigned int i;
    unsigned char ix=0,i1,i2;
    switch (beeptype) {
        case BEEP_TIC:
            for (i = 0; i < 50; i++) { // 20ms
                DAC1CON1 = 0;
                __delay_us(200); // about 2kHz
                DAC1CON1 = 128;
                __delay_us(200); // about 2kHz

            }
            break;
        case BEEP_TIC_FINE:
            for (i = 0; i < 66; i++) { // 20ms
                DAC1CON1 = 0;
                __delay_us(130); // about 3kHz
                DAC1CON1 = 128;
                __delay_us(130); // about 3kHz

            }
            break;
        case BEEP_3TICS:
            for (i1 = 0; i1 < 3; i1++) {
                for (i2 = 0; i2 < 33; i2++) { // 10ms
                    //TOGGLE_RA0 (1500);       // 3kHz
                }
                __delay_ms(150);
            }
            break;
        case BEEP_SHORT_SIN:
            for (i = 0; i < 4000; i++) { // 130ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(30); // about 1kHz
            }
            break;
        case BEEP_LONG_SIN:
            for (i = 0; i < 12000; i++) { // 400ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(30); // about 1kHz
            }
            break;
        case BEEP_WELCOME:
            for (i = 0; i < 5000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(20); // about 1.5kHz
            }
            for (i = 0; i < 5000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(15); // about 1.7kHz
            }
            for (i = 0; i < 5000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(10); // about 3kHz
            }
            break;
        case BEEP_UP:
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(20); // about 1.5kHz
            }
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(15); // about 1.7kHz
            }
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(10); // about 3kHz
            }
            break;
       case BEEP_DOWN:
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(10); // about 3kHz
            }
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(15); // about 1.7kHz
            }
            for (i = 0; i < 1000; i++) { // 500ms
                DAC1CON1 = SINTAB[(ix++) & 0x1F];
                __delay_us(20); // about 1.5kHz
            }
            break;
    }

    PORTAbits.RA0 = 0; // Set RA0 to 0 to save energy
}

static void SND_MorseCode(unsigned char number) {
    unsigned char i;

    for (i = 0; i < 5; i++) {
        if (number <= 5) {
            if (i < number)
                SND_Beep(BEEP_SHORT_SIN);
            else
                SND_Beep(BEEP_LONG_SIN);
        } else {
            if (i < (number - 5))
                SND_Beep(BEEP_LONG_SIN);
            else
                SND_Beep(BEEP_SHORT_SIN);
        }
        __delay_ms(200);
    }



}

void SND_Morse(unsigned int number) {

    unsigned int n = number;
    unsigned char x;
    unsigned int digits = 1000;
    unsigned char nonZ = 0;

    // Split the number into single digits
    while (digits) {
        x = (n / digits);
        if (x > 0)
            nonZ = 1;
        n = n % digits;

        // Output without trailing zeros
        if (nonZ) {
            SND_MorseCode(x);
            if (digits > 1)
                __delay_ms(800);
        }

        digits /= 10;
    }


}